@@ -35,7 +35,7 @@ from PyQt5.QtWidgets import QApplication
 from PyQt5.QtWebKit import QWebSettings
 from PyQt5.QtGui import QDesktopServices, QPixmap, QIcon, QCursor, QWindow
 from PyQt5.QtCore import (pyqtSlot, qInstallMessageHandler, QTimer, QUrl,
-                          QObject, Qt, QEvent)
+                          QObject, Qt, QEvent, pyqtSignal)
 try:
     import hunter
 except ImportError:
@@ -165,11 +165,14 @@ def init(args, crash_handler):
 def _init_icon():
     """Initialize the icon of qutebrowser."""
     icon = QIcon()
+    fallback_icon = QIcon()
     for size in (16, 24, 32, 48, 64, 96, 128, 256, 512):
         filename = ':/icons/qutebrowser-{}x{}.png'.format(size, size)
         pixmap = QPixmap(filename)
         qtutils.ensure_not_null(pixmap)
-        icon.addPixmap(pixmap)
+        fallback_icon.addPixmap(pixmap)
+    qtutils.ensure_not_null(fallback_icon)
+    icon = QIcon.fromTheme('qutebrowser', fallback_icon)
     qtutils.ensure_not_null(icon)
     qApp.setWindowIcon(icon)
 
@@ -257,7 +260,7 @@ def process_pos_args(args, via_ipc=False, cwd=None, target_arg=None):
         if cmd.startswith(':'):
             if win_id is None:
                 win_id = mainwindow.get_window(via_ipc, force_tab=True)
-            log.init.debug("Startup cmd {}".format(cmd))
+            log.init.debug("Startup cmd {!r}".format(cmd))
             commandrunner = runners.CommandRunner(win_id)
             commandrunner.run_safely_init(cmd[1:])
         elif not cmd:
@@ -550,6 +553,10 @@ class Quitter:
         else:
             argdict['session'] = session
             argdict['override_restore'] = False
+        # Ensure :restart works with --temp-basedir
+        argdict['temp_basedir'] = False
+        argdict['temp_basedir_restarted'] = True
+
         # Dump the data
         data = json.dumps(argdict)
         args += ['--json-args', data]
@@ -572,7 +579,7 @@ class Quitter:
             raise cmdexc.CommandError("SyntaxError in {}:{}: {}".format(
                 e.filename, e.lineno, e))
         if ok:
-            self.shutdown()
+            self.shutdown(restart=True)
 
     def restart(self, pages=(), session=None):
         """Inner logic to restart qutebrowser.
@@ -615,7 +622,8 @@ class Quitter:
 
     @cmdutils.register(instance='quitter', name=['quit', 'q'],
                        ignore_args=True)
-    def shutdown(self, status=0, session=None, last_window=False):
+    def shutdown(self, status=0, session=None, last_window=False,
+                 restart=False):
         """Quit qutebrowser.
 
         Args:
@@ -623,6 +631,7 @@ class Quitter:
             session: A session name if saving should be forced.
             last_window: If the shutdown was triggered due to the last window
                             closing.
+            restart: If we're planning to restart.
         """
         if self._shutting_down:
             return
@@ -652,13 +661,14 @@ class Quitter:
             # in the real main event loop, or we'll get a segfault.
             log.destroy.debug("Deferring real shutdown because question was "
                               "active.")
-            QTimer.singleShot(0, functools.partial(self._shutdown, status))
+            QTimer.singleShot(0, functools.partial(self._shutdown, status,
+                                                   restart=restart))
         else:
             # If we have no questions to shut down, we are already in the real
             # event loop, so we can shut down immediately.
-            self._shutdown(status)
+            self._shutdown(status, restart=restart)
 
-    def _shutdown(self, status):
+    def _shutdown(self, status, restart):
         """Second stage of shutdown."""
         log.destroy.debug("Stage 2 of shutting down...")
         if qApp is None:
@@ -699,7 +709,8 @@ class Quitter:
         log.destroy.debug("Deactivating crash log...")
         objreg.get('crash-handler').destroy_crashlogfile()
         # Delete temp basedir
-        if self._args.temp_basedir:
+        if ((self._args.temp_basedir or self._args.temp_basedir_restarted) and
+                not restart):
             atexit.register(shutil.rmtree, self._args.basedir,
                             ignore_errors=True)
         # If we don't kill our custom handler here we might get segfaults
@@ -731,6 +742,8 @@ class Application(QApplication):
         _args: ArgumentParser instance.
     """
 
+    new_window = pyqtSignal(mainwindow.MainWindow)
+
     def __init__(self, args):
         """Constructor.
 
