@@ -30,7 +30,7 @@ import xml.etree.ElementTree
 from PyQt5.QtWebKit import QWebSettings
 from PyQt5.QtWidgets import QApplication, QTabBar
 from PyQt5.QtCore import Qt, QUrl, QEvent
-from PyQt5.QtGui import QClipboard, QKeyEvent
+from PyQt5.QtGui import QKeyEvent
 from PyQt5.QtPrintSupport import QPrintDialog, QPrintPreviewDialog
 from PyQt5.QtWebKitWidgets import QWebPage
 import pygments
@@ -45,6 +45,7 @@ from qutebrowser.utils import (message, usertypes, log, qtutils, urlutils,
                                objreg, utils)
 from qutebrowser.utils.usertypes import KeyMode
 from qutebrowser.misc import editor, guiprocess
+from qutebrowser.completion.models import instances, sortfilter
 
 
 class CommandDispatcher:
@@ -659,7 +660,6 @@ class CommandDispatcher:
             title: Yank the title instead of the URL.
             domain: Yank only the scheme, domain, and port number.
         """
-        clipboard = QApplication.clipboard()
         if title:
             s = self._tabbed_browser.page_title(self._current_index())
             what = 'title'
@@ -673,23 +673,16 @@ class CommandDispatcher:
             s = self._current_url().toString(
                 QUrl.FullyEncoded | QUrl.RemovePassword)
             what = 'URL'
-        if sel and clipboard.supportsSelection():
-            mode = QClipboard.Selection
+
+        if sel and QApplication.clipboard().supportsSelection():
             target = "primary selection"
         else:
-            mode = QClipboard.Clipboard
+            sel = False
             target = "clipboard"
-        log.misc.debug("Yanking to {}: '{}'".format(target, s))
 
-        msg = "Yanked {} to {}: {}".format(what, target, s)
-        clipboard.changed.connect(functools.partial(
-            self._display_yank_msg, clipboard, msg))
-        clipboard.setText(s, mode)
-
-    def _display_yank_msg(self, clipboard, msg):
-        """Display a message when something was yanked."""
-        message.info(self._win_id, msg)
-        clipboard.changed.disconnect()
+        utils.set_clipboard(s, selection=sel)
+        message.info(self._win_id, "Yanked {} to {}: {}".format(
+                     what, target, s))
 
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        count='count')
@@ -809,28 +802,92 @@ class CommandDispatcher:
     def paste(self, sel=False, tab=False, bg=False, window=False):
         """Open a page from the clipboard.
 
+        If the pasted text contains newlines, each line gets opened in its own
+        tab.
+
         Args:
             sel: Use the primary selection instead of the clipboard.
             tab: Open in a new tab.
             bg: Open in a background tab.
             window: Open in new window.
         """
-        clipboard = QApplication.clipboard()
-        if sel and clipboard.supportsSelection():
-            mode = QClipboard.Selection
+        if sel and QApplication.clipboard().supportsSelection():
             target = "Primary selection"
         else:
-            mode = QClipboard.Clipboard
+            sel = False
             target = "Clipboard"
-        text = clipboard.text(mode)
-        if not text:
+        text = utils.get_clipboard(selection=sel)
+        if not text.strip():
             raise cmdexc.CommandError("{} is empty.".format(target))
-        log.misc.debug("{} contained: '{}'".format(target, text))
+        log.misc.debug("{} contained: {!r}".format(target, text))
+        text_urls = [u for u in text.split('\n') if u.strip()]
+        if (len(text_urls) > 1 and not urlutils.is_url(text_urls[0]) and
+            urlutils.get_path_if_valid(
+                text_urls[0], check_exists=True) is None):
+            text_urls = [text]
+        for i, text_url in enumerate(text_urls):
+            if not window and i > 0:
+                tab = False
+                bg = True
+            try:
+                url = urlutils.fuzzy_url(text_url)
+            except urlutils.InvalidUrlError as e:
+                raise cmdexc.CommandError(e)
+            self._open(url, tab, bg, window)
+
+    @cmdutils.register(instance='command-dispatcher', scope='window',
+                       completion=[usertypes.Completion.tab])
+    def buffer(self, index):
+        """Select tab by index or url/title best match.
+
+        Focuses window if necessary.
+
+        Args:
+            index: The [win_id/]index of the tab to focus. Or a substring
+                   in which case the closest match will be focused.
+        """
+        index_parts = index.split('/', 1)
+
         try:
-            url = urlutils.fuzzy_url(text)
-        except urlutils.InvalidUrlError as e:
-            raise cmdexc.CommandError(e)
-        self._open(url, tab, bg, window)
+            for part in index_parts:
+                int(part)
+        except ValueError:
+            model = instances.get(usertypes.Completion.tab)
+            sf = sortfilter.CompletionFilterModel(source=model)
+            sf.set_pattern(index)
+            if sf.count() > 0:
+                index = sf.data(sf.first_item())
+                index_parts = index.split('/', 1)
+            else:
+                raise cmdexc.CommandError(
+                    "No matching tab for: {}".format(index))
+
+        if len(index_parts) == 2:
+            win_id = int(index_parts[0])
+            idx = int(index_parts[1])
+        elif len(index_parts) == 1:
+            idx = int(index_parts[0])
+            active_win = objreg.get('app').activeWindow()
+            if active_win is None:
+                # Not sure how you enter a command without an active window...
+                raise cmdexc.CommandError(
+                    "No window specified and couldn't find active window!")
+            win_id = active_win.win_id
+
+        if win_id not in objreg.window_registry:
+            raise cmdexc.CommandError(
+                "There's no window with id {}!".format(win_id))
+
+        tabbed_browser = objreg.get('tabbed-browser', scope='window',
+                                    window=win_id)
+        if not 0 < idx <= tabbed_browser.count():
+            raise cmdexc.CommandError(
+                "There's no tab with index {}!".format(idx))
+
+        window = objreg.window_registry[win_id]
+        window.activateWindow()
+        window.raise_()
+        tabbed_browser.setCurrentIndex(idx-1)
 
     @cmdutils.register(instance='command-dispatcher', scope='window',
                        count='count')
@@ -916,9 +973,12 @@ class CommandDispatcher:
         useful here.
 
         Args:
-            userscript: Run the command as a userscript. Either store the
-                        userscript in `~/.local/share/qutebrowser/userscripts`
-                        (or `$XDG_DATA_DIR`), or use an absolute path.
+            userscript: Run the command as a userscript. You can use an
+                        absolute path, or store the userscript in one of those
+                        locations:
+                            - `~/.local/share/qutebrowser/userscripts`
+                              (or `$XDG_DATA_DIR`)
+                            - `/usr/share/qutebrowser/userscripts`
             verbose: Show notifications when the command started/exited.
             detach: Whether the command should be detached from qutebrowser.
             cmdline: The commandline to execute.
@@ -1298,6 +1358,33 @@ class CommandDispatcher:
         except webelem.IsNullError:
             raise cmdexc.CommandError("Element vanished while editing!")
 
+    @cmdutils.register(instance='command-dispatcher',
+                       modes=[KeyMode.insert], hide=True, scope='window',
+                       needs_js=True)
+    def paste_primary(self):
+        """Paste the primary selection at cursor position."""
+        frame = self._current_widget().page().currentFrame()
+        try:
+            elem = webelem.focus_elem(frame)
+        except webelem.IsNullError:
+            raise cmdexc.CommandError("No element focused!")
+        if not elem.is_editable(strict=True):
+            raise cmdexc.CommandError("Focused element is not editable!")
+
+        try:
+            sel = utils.get_clipboard(selection=True)
+        except utils.SelectionUnsupportedError:
+            return
+
+        log.misc.debug("Pasting primary selection into element {}".format(
+            elem.debug_text()))
+        elem.evaluateJavaScript("""
+            var sel = '{}';
+            var event = document.createEvent('TextEvent');
+            event.initTextEvent('textInput', true, true, null, sel);
+            this.dispatchEvent(event);
+        """.format(webelem.javascript_escape(sel)))
+
     def _clear_search(self, view, text):
         """Clear search string/highlights for the given view.
 
@@ -1462,11 +1549,11 @@ class CommandDispatcher:
         webview = self._current_widget()
         if not webview.selection_enabled:
             act = [QWebPage.MoveToNextWord]
-            if sys.platform == 'win32':
+            if sys.platform == 'win32':  # pragma: no cover
                 act.append(QWebPage.MoveToPreviousChar)
         else:
             act = [QWebPage.SelectNextWord]
-            if sys.platform == 'win32':
+            if sys.platform == 'win32':  # pragma: no cover
                 act.append(QWebPage.SelectPreviousChar)
         for _ in range(count):
             for a in act:
@@ -1483,11 +1570,11 @@ class CommandDispatcher:
         webview = self._current_widget()
         if not webview.selection_enabled:
             act = [QWebPage.MoveToNextWord]
-            if sys.platform != 'win32':
+            if sys.platform != 'win32':  # pragma: no branch
                 act.append(QWebPage.MoveToNextChar)
         else:
             act = [QWebPage.SelectNextWord]
-            if sys.platform != 'win32':
+            if sys.platform != 'win32':  # pragma: no branch
                 act.append(QWebPage.SelectNextChar)
         for _ in range(count):
             for a in act:
@@ -1640,14 +1727,12 @@ class CommandDispatcher:
             message.info(self._win_id, "Nothing to yank")
             return
 
-        clipboard = QApplication.clipboard()
-        if sel and clipboard.supportsSelection():
-            mode = QClipboard.Selection
+        if sel and QApplication.clipboard().supportsSelection():
             target = "primary selection"
         else:
-            mode = QClipboard.Clipboard
+            sel = False
             target = "clipboard"
-        clipboard.setText(s, mode)
+        utils.set_clipboard(s, sel)
         message.info(self._win_id, "{} {} yanked to {}".format(
             len(s), "char" if len(s) == 1 else "chars", target))
         if not keep:
@@ -1755,3 +1840,10 @@ class CommandDispatcher:
 
             QApplication.postEvent(receiver, press_event)
             QApplication.postEvent(receiver, release_event)
+
+    @cmdutils.register(instance='command-dispatcher', scope='window',
+                       debug=True)
+    def debug_clear_ssl_errors(self):
+        """Clear remembered SSL error answers."""
+        nam = self._current_widget().page().networkAccessManager()
+        nam.clear_all_ssl_errors()
