@@ -122,7 +122,10 @@ class BrowserPage(QWebPage):
         """
         ignored_errors = [
             (QWebPage.QtNetwork, QNetworkReply.OperationCanceledError),
-            (QWebPage.WebKit, 203),  # "Loading is handled by the media engine"
+            # "Loading is handled by the media engine"
+            (QWebPage.WebKit, 203),
+            # "Frame load interrupted by policy change"
+            (QWebPage.WebKit, 102),
         ]
         errpage.baseUrl = info.url
         urlstr = info.url.toDisplayString()
@@ -166,10 +169,8 @@ class BrowserPage(QWebPage):
             log.webview.debug("Error domain: {}, error code: {}".format(
                 info.domain, info.error))
             title = "Error loading page: {}".format(urlstr)
-            template = jinja.env.get_template('error.html')
-            # pylint: disable=no-member
-            # WORKAROUND for https://bitbucket.org/logilab/pylint/issue/490/
-            html = template.render(
+            html = jinja.render(
+                'error.html',
                 title=title, url=urlstr, error=error_str, icon='')
             errpage.content = html.encode('utf-8')
             errpage.encoding = 'utf-8'
@@ -221,14 +222,12 @@ class BrowserPage(QWebPage):
     def _show_pdfjs(self, reply):
         """Show the reply with pdfjs."""
         try:
-            page = pdfjs.generate_pdfjs_page(reply.url()).encode('utf-8')
+            page = pdfjs.generate_pdfjs_page(reply.url())
         except pdfjs.PDFJSNotFound:
-            # pylint: disable=no-member
-            # WORKAROUND for https://bitbucket.org/logilab/pylint/issue/490/
-            page = (jinja.env.get_template('no_pdfjs.html')
-                    .render(url=reply.url().toDisplayString())
-                    .encode('utf-8'))
-        self.mainFrame().setContent(page, 'text/html', reply.url())
+            page = jinja.render('no_pdfjs.html',
+                                url=reply.url().toDisplayString())
+        self.mainFrame().setContent(page.encode('utf-8'), 'text/html',
+                                    reply.url())
         reply.deleteLater()
 
     def shutdown(self):
@@ -372,6 +371,7 @@ class BrowserPage(QWebPage):
             q.answered_no.connect(no_action)
             q.cancelled.connect(no_action)
 
+            self.shutting_down.connect(q.abort)
             q.completed.connect(q.deleteLater)
 
             self.featurePermissionRequestCanceled.connect(functools.partial(
