@@ -863,12 +863,14 @@ def data(readonly=False):
                  valid_values=typ.ValidValues(
                      ('number', "Use numeric hints."),
                      ('letter', "Use the chars in the hints -> "
-                      "chars setting.")
+                      "chars setting."),
+                     ('word', "Use hints words based on the html "
+                      "elements and the extra words."),
                  )), 'letter'),
              "Mode to use for hints."),
 
             ('chars',
-             SettingValue(typ.String(minlen=2, completions=[
+             SettingValue(typ.UniqueCharString(minlen=2, completions=[
                  ('asdfghjkl', "Home row"),
                  ('dhtnaoeu', "Home row (Dvorak)"),
                  ('abcdefghijklmnopqrstuvwxyz', "All letters"),
@@ -888,9 +890,14 @@ def data(readonly=False):
              SettingValue(typ.Bool(), 'false'),
              "Make chars in hint strings uppercase."),
 
+            ('dictionary',
+             SettingValue(typ.File(required=False), '/usr/share/dict/words'),
+             "The dictionary file to be used by the word hints."),
+
             ('auto-follow',
              SettingValue(typ.Bool(), 'true'),
-             "Whether to auto-follow a hint if there's only one left."),
+             "Follow a hint immediately when the hint text is completely "
+             "matched."),
 
             ('next-regexes',
              SettingValue(typ.RegexList(flags=re.IGNORECASE),
@@ -1193,7 +1200,7 @@ def data(readonly=False):
              SettingValue(typ.Font(), 'Terminus, Monospace, '
                           '"DejaVu Sans Mono", Monaco, '
                           '"Bitstream Vera Sans Mono", "Andale Mono", '
-                          '"Liberation Mono", "Courier New", Courier, '
+                          '"Courier New", Courier, "Liberation Mono", '
                           'monospace, Fixed, Consolas, Terminal'),
              "Default monospace fonts."),
 
@@ -1324,7 +1331,8 @@ KEY_SECTION_DESC = {
         "Since normal keypresses are passed through, only special keys are "
         "supported in this mode.\n"
         "Useful hidden commands to map in this section:\n\n"
-        " * `open-editor`: Open a texteditor with the focused field."),
+        " * `open-editor`: Open a texteditor with the focused field.\n"
+        " * `paste-primary`: Paste primary selection at cursor position."),
     'hint': (
         "Keybindings for hint mode.\n"
         "Since normal keypresses are passed through, only special keys are "
@@ -1389,8 +1397,8 @@ KEY_DATA = collections.OrderedDict([
         ('tab-move', ['gm']),
         ('tab-move -', ['gl']),
         ('tab-move +', ['gr']),
-        ('tab-focus', ['J', 'gt']),
-        ('tab-prev', ['K', 'gT']),
+        ('tab-focus', ['J']),
+        ('tab-prev', ['K']),
         ('tab-clone', ['gC']),
         ('reload', ['r']),
         ('reload -f', ['R']),
@@ -1469,6 +1477,7 @@ KEY_DATA = collections.OrderedDict([
         ('download-cancel', ['ad']),
         ('download-clear', ['cd']),
         ('view-source', ['gf']),
+        ('set-cmd-text -s :buffer', ['gt']),
         ('tab-focus last', ['<Ctrl-Tab>']),
         ('enter-mode passthrough', ['<Ctrl-V>']),
         ('quit', ['<Ctrl-Q>']),
@@ -1495,6 +1504,7 @@ KEY_DATA = collections.OrderedDict([
 
     ('insert', collections.OrderedDict([
         ('open-editor', ['<Ctrl-E>']),
+        ('paste-primary', ['<Shift-Ins>']),
     ])),
 
     ('hint', collections.OrderedDict([
