@@ -283,6 +283,21 @@ class String(BaseType):
             return super().complete()
 
 
+class UniqueCharString(String):
+
+    """A string which may not contain duplicate chars."""
+
+    def validate(self, value):
+        super().validate(value)
+        if not value:
+            return
+
+        # Check for duplicate values
+        if len(set(value)) != len(value):
+            raise configexc.ValidationError(
+                value, "String contains duplicate values!")
+
+
 class List(BaseType):
 
     """Base class for a (string-)list setting."""
@@ -901,6 +916,10 @@ class File(BaseType):
 
     """A file on the local filesystem."""
 
+    def __init__(self, required=True, **kwargs):
+        super().__init__(**kwargs)
+        self.required = required
+
     def transform(self, value):
         if not value:
             return None
@@ -909,7 +928,7 @@ class File(BaseType):
         if not os.path.isabs(value):
             cfgdir = standarddir.config()
             assert cfgdir is not None
-            return os.path.join(cfgdir, value)
+            value = os.path.join(cfgdir, value)
         return value
 
     def validate(self, value):
@@ -925,15 +944,13 @@ class File(BaseType):
                     raise configexc.ValidationError(
                         value, "must be an absolute path when not using a "
                         "config directory!")
-                elif not os.path.isfile(os.path.join(cfgdir, value)):
-                    raise configexc.ValidationError(
-                        value, "must be a valid path relative to the config "
-                        "directory!")
-                else:
-                    return
-            elif not os.path.isfile(value):
-                raise configexc.ValidationError(
-                    value, "must be a valid file!")
+                value = os.path.join(cfgdir, value)
+                not_isfile_message = ("must be a valid path relative to the "
+                                      "config directory!")
+            else:
+                not_isfile_message = "must be a valid file!"
+            if self.required and not os.path.isfile(value):
+                raise configexc.ValidationError(value, not_isfile_message)
         except UnicodeEncodeError as e:
             raise configexc.ValidationError(value, e)
 
@@ -1094,7 +1111,7 @@ class ShellCommand(BaseType):
             shlex.split(value)
         except ValueError as e:
             raise configexc.ValidationError(value, str(e))
-        if self.placeholder and '{}' not in self.transform(value):
+        if self.placeholder and '{}' not in value:
             raise configexc.ValidationError(value, "needs to contain a "
                                             "{}-placeholder.")
 
@@ -1179,7 +1196,7 @@ class SearchEngineUrl(BaseType):
         self._basic_validation(value)
         if not value:
             return
-        elif '{}' not in value:
+        elif not ('{}' in value or '{0}' in value):
             raise configexc.ValidationError(value, "must contain \"{}\"")
         try:
             value.format("")
@@ -1269,8 +1286,16 @@ class UserStyleSheet(File):
     def transform(self, value):
         if not value:
             return None
-        path = super().transform(value)
-        if os.path.exists(path):
+
+        if standarddir.config() is None:
+            # We can't call super().transform() here as this counts on the
+            # validation previously ensuring that we don't have a relative path
+            # when starting with -c "".
+            path = None
+        else:
+            path = super().transform(value)
+
+        if path is not None and os.path.exists(path):
             return QUrl.fromLocalFile(path)
         else:
             data = base64.b64encode(value.encode('utf-8')).decode('ascii')
