@@ -63,7 +63,7 @@ class TabbedBrowser(tabwidget.TabWidget):
                          tabbar -> new-tab-position set to 'left'.
         _tab_insert_idx_right: Same as above, for 'right'.
         _undo_stack: List of UndoEntry namedtuples of closed tabs.
-        _shutting_down: Whether we're currently shutting down.
+        shutting_down: Whether we're currently shutting down.
 
     Signals:
         cur_progress: Progress of the current tab changed (loadProgress).
@@ -82,6 +82,7 @@ class TabbedBrowser(tabwidget.TabWidget):
                  widget can adjust its size to it.
                  arg: The new size.
         current_tab_changed: The current tab changed to the emitted WebView.
+        new_tab: Emits the new WebView and its index when a new tab is opened.
     """
 
     cur_progress = pyqtSignal(int)
@@ -96,13 +97,14 @@ class TabbedBrowser(tabwidget.TabWidget):
     resized = pyqtSignal('QRect')
     got_cmd = pyqtSignal(str)
     current_tab_changed = pyqtSignal(webview.WebView)
+    new_tab = pyqtSignal(webview.WebView, int)
 
     def __init__(self, win_id, parent=None):
         super().__init__(win_id, parent)
         self._win_id = win_id
         self._tab_insert_idx_left = 0
         self._tab_insert_idx_right = -1
-        self._shutting_down = False
+        self.shutting_down = False
         self.tabCloseRequested.connect(self.on_tab_close_requested)
         self.currentChanged.connect(self.on_current_changed)
         self.cur_load_started.connect(self.on_cur_load_started)
@@ -234,7 +236,7 @@ class TabbedBrowser(tabwidget.TabWidget):
 
     def shutdown(self):
         """Try to shut down all tabs cleanly."""
-        self._shutting_down = True
+        self.shutting_down = True
         for tab in self.widgets():
             self._remove_tab(tab)
 
@@ -272,8 +274,8 @@ class TabbedBrowser(tabwidget.TabWidget):
         """
         idx = self.indexOf(tab)
         if idx == -1:
-            raise ValueError("tab {} is not contained in TabbedWidget!".format(
-                tab))
+            raise TabDeletedError("tab {} is not contained in "
+                                  "TabbedWidget!".format(tab))
         if tab is self._now_focused:
             self._now_focused = None
         if tab is objreg.get('last-focused-tab', None, scope='window',
@@ -350,7 +352,11 @@ class TabbedBrowser(tabwidget.TabWidget):
     @pyqtSlot(webview.WebView)
     def on_window_close_requested(self, widget):
         """Close a tab with a widget given."""
-        self.close_tab(widget)
+        try:
+            self.close_tab(widget)
+        except TabDeletedError:
+            log.webview.debug("Requested to close {!r} which does not "
+                              "exist!".format(widget))
 
     @pyqtSlot('QUrl', bool)
     def tabopen(self, url=None, background=None, explicit=False):
@@ -394,6 +400,7 @@ class TabbedBrowser(tabwidget.TabWidget):
         if not background:
             self.setCurrentWidget(tab)
         tab.show()
+        self.new_tab.emit(tab, idx)
         return tab
 
     def _get_new_tab_idx(self, explicit):
@@ -542,7 +549,7 @@ class TabbedBrowser(tabwidget.TabWidget):
     @pyqtSlot(int)
     def on_current_changed(self, idx):
         """Set last-focused-tab and leave hinting mode when focus changed."""
-        if idx == -1 or self._shutting_down:
+        if idx == -1 or self.shutting_down:
             # closing the last tab (before quitting) or shutting down
             return
         tab = self.widget(idx)
@@ -609,7 +616,7 @@ class TabbedBrowser(tabwidget.TabWidget):
     def on_scroll_pos_changed(self):
         """Update tab and window title when scroll position changed."""
         self.update_window_title()
-        self.update_tab_titles()
+        self.update_tab_title(self.currentIndex())
 
     def resizeEvent(self, e):
         """Extend resizeEvent of QWidget to emit a resized signal afterwards.
