@@ -51,10 +51,11 @@ else:
         colorama.deinit()
 
 # Log formats to use.
-SIMPLE_FMT = '{levelname}: {message}'
+SIMPLE_FMT = '{asctime:8} {levelname}: {message}'
 EXTENDED_FMT = ('{asctime:8} {levelname:8} {name:10} {module}:{funcName}:'
                 '{lineno} {message}')
-SIMPLE_FMT_COLORED = '%(log_color)s%(levelname)s%(reset)s: %(message)s'
+SIMPLE_FMT_COLORED = ('%(green)s%(asctime)-8s%(reset)s '
+                      '%(log_color)s%(levelname)s%(reset)s: %(message)s')
 EXTENDED_FMT_COLORED = (
     '%(green)s%(asctime)-8s%(reset)s '
     '%(log_color)s%(levelname)-8s%(reset)s '
@@ -274,7 +275,7 @@ def qt_message_handler(msg_type, context, msg):
         # PNGs in Qt with broken color profile
         # https://bugreports.qt.io/browse/QTBUG-39788
         'libpng warning: iCCP: Not recognizing known sRGB profile that has '
-            'been edited',  # noqa
+            'been edited',  # flake8: disable=E131
         'libpng warning: iCCP: known incorrect sRGB profile',
         # Hopefully harmless warning
         'OpenType support missing for script ',
@@ -294,6 +295,8 @@ def qt_message_handler(msg_type, context, msg):
         # Hopefully harmless
         '"Method "GetAll" with signature "s" on interface '
             '"org.freedesktop.DBus.Properties" doesn\'t exist',
+        '"Method \\"GetAll\\" with signature \\"s\\" on interface '
+            '\\"org.freedesktop.DBus.Properties\\" doesn\'t exist\\n"',
         'WOFF support requires QtWebKit to be built with zlib support.',
         # Weird Enlightment/GTK X extensions
         'QXcbWindow: Unhandled client message: "_E_',
@@ -312,7 +315,7 @@ def qt_message_handler(msg_type, context, msg):
             'libpng warning: iCCP: known incorrect sRGB profile',
             # https://bugreports.qt.io/browse/QTBUG-47154
             'virtual void QSslSocketBackendPrivate::transmit() SSLRead failed '
-                'with: -9805',  # noqa
+                'with: -9805',  # flake8: disable=E131
         ]
 
     # Messages which will trigger an exception immediately
